import React, { useState } from 'react';
import { Button, Box, Typography, IconButton } from '@mui/material';
import { Add as AddIcon, Delete as DeleteIcon, UploadFile as UploadFileIcon, Refresh as RefreshIcon } from '@mui/icons-material';
import UploadDialog from './UploadDialog';
import { Dewar, ShipmentsService } from '../../openapi';
import { SxProps } from '@mui/material';
import bottleGrey from '/src/assets/icons/bottle-svgrepo-com-grey.svg';
import bottleYellow from '/src/assets/icons/bottle-svgrepo-com-yellow.svg';
import bottleGreen from '/src/assets/icons/bottle-svgrepo-com-green.svg';
import bottleRed from '/src/assets/icons/bottle-svgrepo-com-red.svg';

interface ShipmentPanelProps {
    setCreatingShipment: (value: boolean) => void;
    selectShipment: (shipment: ShipmentsService | null) => void;
    selectedShipment: ShipmentsService | null;
    sx?: SxProps;
    shipments: ShipmentsService[];
    refreshShipments: () => void;
    error: string | null;
}

const statusIconMap: Record<string, string> = {
    'In Transit': bottleYellow,
    'Delivered': bottleGreen,
    'Pending': bottleGrey,
    'Unknown': bottleRed,
};

const ShipmentPanel: React.FC<ShipmentPanelProps> = ({
                                                         setCreatingShipment,
                                                         selectShipment,
                                                         selectedShipment,
                                                         sx,
                                                         shipments,
                                                         refreshShipments,
                                                         error,
                                                     }) => {
    const [uploadDialogOpen, setUploadDialogOpen] = useState(false);

    const handleDeleteShipment = async () => {
        if (selectedShipment) {
            const confirmed = window.confirm(`Are you sure you want to delete the shipment: ${selectedShipment.shipment_name}?`);
            if (confirmed) {
                await deleteShipment(selectedShipment.shipment_id);
            }
        }
    };

    const deleteShipment = async (shipmentId: string | undefined) => {
        if (!shipmentId) return;
        try {
            await ShipmentsService.deleteShipmentShipmentsShipmentIdDelete(shipmentId);
            refreshShipments();
            selectShipment(null);
        } catch (error) {
            console.error('Failed to delete shipment:', error);
        }
    };

    const handleShipmentSelection = (shipment: ShipmentsService) => {
        const isSelected = selectedShipment?.shipment_id === shipment.shipment_id;
        const updatedShipment = isSelected ? null : shipment;
        console.log("Shipment selected:", updatedShipment); // debug log
        selectShipment(updatedShipment);
    };

    const openUploadDialog = (event: React.MouseEvent) => {
        event.stopPropagation(); // Prevent event bubbling
        setUploadDialogOpen(true);
    };

    const closeUploadDialog = () => setUploadDialogOpen(false);

    const getNumberOfDewars = (shipment: ShipmentsService): number => shipment.dewars?.length || 0;

    console.log("Current selected shipment:", selectedShipment); // debug log

    return (
        <Box sx={{ width: '90%', borderRight: '1px solid #ccc', padding: 2, ...sx }}>
            {error && <Typography color="error">{error}</Typography>}
            <Typography variant="h6" sx={{ marginBottom: 2, fontWeight: 'bold' }}>
                Shipments
            </Typography>
            <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 2 }}>
                <Button
                    variant="contained"
                    color="primary"
                    startIcon={<AddIcon />}
                    onClick={() => setCreatingShipment(true)}
                    sx={{ padding: '10px 16px' }}
                >
                    Create Shipment
                </Button>
                <IconButton
                    onClick={refreshShipments}
                    color="primary"
                    title="Refresh Shipments"
                    sx={{ marginLeft: 1 }}
                >
                    <RefreshIcon />
                </IconButton>
            </Box>
            {shipments.map((shipment) => (
                <Button
                    key={shipment.shipment_id}
                    onClick={() => handleShipmentSelection(shipment)}
                    sx={{
                        width: '100%',
                        textAlign: 'left',
                        marginBottom: 1,
                        color: 'white',
                        whiteSpace: 'nowrap',
                        overflow: 'hidden',
                        textOverflow: 'ellipsis',
                        padding: '10px 16px',
                        fontSize: '0.7rem',
                        display: 'flex',
                        alignItems: 'center',
                        justifyContent: 'space-between',
                        backgroundColor: selectedShipment?.shipment_id === shipment.shipment_id ? '#52893e' : '#424242',
                        '&:hover': {
                            backgroundColor: selectedShipment?.shipment_id === shipment.shipment_id ? '#9aca8c' : '#616161',
                        },
                        '&:active': {
                            backgroundColor: selectedShipment?.shipment_id === shipment.shipment_id ? '#915151' : '#212121',
                        },
                    }}
                >
                    <Box sx={{ display: 'flex', alignItems: 'center' }}>
                        <Box sx={{ position: 'relative', marginRight: '8px' }}>
                            <img
                                src={statusIconMap[shipment.shipment_status] || bottleGrey}
                                alt={`Status: ${shipment.shipment_status}`}
                                width="24"
                            />
                            <Typography
                                component="span"
                                sx={{
                                    position: 'absolute',
                                    top: '0%',
                                    right: '0%',
                                    transform: 'translate(50%, -50%)',
                                    color: 'white',
                                    fontWeight: 'bold',
                                    fontSize: '0.6rem',
                                    backgroundColor: 'transparent',
                                    borderRadius: '50%',
                                    padding: '0 2px',
                                }}
                            >
                                {getNumberOfDewars(shipment)}
                            </Typography>
                        </Box>
                        <Box>
                            <Typography>{shipment.shipment_name}</Typography>
                            <Typography sx={{ fontSize: '0.6rem', color: '#ccc' }}>{shipment.shipment_date}</Typography>
                            <Typography sx={{ fontSize: '0.6rem', color: '#ccc' }}>
                                Total Pucks: {shipment.dewars.reduce((total, dewar: Dewar) => total + dewar.number_of_pucks, 0)}
                            </Typography>
                        </Box>
                    </Box>
                    <Box sx={{ display: 'flex', alignItems: 'center' }}>
                        {selectedShipment?.shipment_id === shipment.shipment_id && (
                            <>
                                <IconButton
                                    onClick={openUploadDialog}
                                    color="primary"
                                    title="Upload Sample Data Sheet"
                                    sx={{ marginLeft: 1 }}
                                >
                                    <UploadFileIcon />
                                </IconButton>
                                <IconButton
                                    onClick={(event) => {
                                        event.stopPropagation();
                                        console.log('Delete button clicked'); // debug log
                                        handleDeleteShipment();
                                    }}
                                    color="error"
                                    title="Delete Shipment"
                                    sx={{ marginLeft: 1 }}
                                >
                                    <DeleteIcon />
                                </IconButton>
                            </>
                        )}
                    </Box>
                </Button>
            ))}
            <UploadDialog
                open={uploadDialogOpen}
                onClose={closeUploadDialog}
            />
        </Box>
    );
};

export default ShipmentPanel;